from __future__ import absolute_import

import glob
import os
import shutil
import tempfile
from unittest import TestCase

class DiskTestCase(TestCase):

    def setUp(self):
        prefix = 'tiledb-' + self.__class__.__name__
        self.rootdir = tempfile.mkdtemp(prefix=prefix)

    def tearDown(self):
        # Remove every directory starting with rootdir
        for dirpath in glob.glob(self.rootdir + "*"):
            try:
                shutil.rmtree(dirpath)
            except OSError as exc:
                print("test '{}' error deleting '{}'".format(self.__class__.__name__,
                                                             dirpath))
                raise

    def path(self, path):
        return os.path.abspath(os.path.join(self.rootdir, path))




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import tiledb
from tiledb import *
from tiledb.libtiledb import index_as_tuple, replace_ellipsis
from tiledb.tests.common import DiskTestCase

import numpy as np
from numpy.testing import assert_equal, assert_approx_equal,\
                          assert_array_equal, assert_raises

import unittest
from unittest import TestCase


class UtilTest(DiskTestCase):
    def test_empty_like(self):
        arr = np.zeros((10,10), dtype=np.float32)

        def check_schema(self, s):
            self.assertEqual(s.attr(0).dtype, np.float32)
            self.assertEqual(s.shape, (10,10))
            self.assertEqual(s.domain.dim(0).shape, (10,))
            self.assertEqual(s.domain.dim(1).shape, (10,))

        with self.assertRaises(ValueError):
            schema_like('', None)

        schema = schema_like(arr, tile=1)
        self.assertIsInstance(schema, ArraySchema)
        check_schema(self, schema)

        uri = self.path("empty_like")
        T = empty_like(uri, arr)
        check_schema(self, T.schema)
        self.assertEqual(T.shape, arr.shape)
        self.assertEqual(T.dtype, arr.dtype)


        # test a fake object with .shape, .ndim, .dtype
        class FakeArray(object):
            def __init__(self, shape, dtype):
                self.shape = shape
                self.ndim = len(shape)
                self.dtype = dtype

        fake = FakeArray((3,3), np.int16)
        schema2 = empty_like(self.path('fake_like'), fake)
        self.assertIsInstance(schema2, Array)
        self.assertEqual(schema2.shape, fake.shape)
        self.assertEqual(schema2.dtype, fake.dtype)
        self.assertEqual(schema2.ndim, fake.ndim)

        # test passing shape and dtype directly
        schema3 = schema_like(shape=(4,4), dtype=np.float32)
        self.assertIsInstance(schema3, ArraySchema)
        self.assertEqual(schema3.attr(0).dtype, np.float32)
        self.assertEqual(schema3.domain.dim(0).tile, 4)
        schema3 = schema_like(shape=(4,4), dtype=np.float32, tile=1)
        self.assertEqual(schema3.domain.dim(0).tile, 1)

    def test_open(self):
        uri = self.path("load")
        with tiledb.from_numpy(uri, np.array(np.arange(3))) as T:
            with tiledb.open(uri) as T2:
                self.assertEqual(T.schema, T2.schema)
                assert_array_equal(T, T2)

    def test_save(self):
        uri = self.path("test_save")
        arr = np.array(np.arange(3))
        with tiledb.save(uri, arr) as tmp:
            with tiledb.open(uri) as T:
                assert_array_equal(arr, T)



import tiledb
from tiledb.libtiledb import *

import numpy as np

def open(uri, key=None, attr=None, mode='r', config=None):
    """
    Open a TileDB array at the given URI

    :param uri: any TileDB supported URI
    :param key: encryption key, str or None
    :param str mode: (default 'r') Open the array object in read 'r' or write 'w' mode
    :param attr: attribute name to select from a multi-attribute array, str or None
    :param config: TileDB config dictionary, dict or None
    :return:
    """
    if config:
        cfg = tiledb.Config(config)
        ctx = tiledb.Ctx(cfg)
    else:
        ctx = default_ctx()

    schema = ArraySchema.load(uri, ctx=ctx)
    if not schema:
        raise Exception("Unable to load tiledb ArraySchema from URI: '{}'".format(uri))

    if schema.sparse:
        return tiledb.SparseArray(uri, mode=mode, key=key, attr=attr, ctx=ctx)
    elif not schema.sparse:
        return tiledb.DenseArray(uri, mode=mode, key=key, attr=attr, ctx=ctx)
    else:
        raise Exception("Unknown TileDB array type")


def save(uri, array, config=None, **kw):
    """
    Save array-like object at the given URI.

    :param uri: str or None
    :param array: array-like object convertible to NumPy
    :param config: TileDB config dictionary, dict or None
    :param kw: optional keyword args will be forwarded to tiledb.Array constructor
    :return:
    """
    if not isinstance(array, np.ndarray):
        raise ValueError("expected NumPy ndarray, not '{}'".format(type(array)))
    if config:
        cfg = Config(config)
        ctx = tiledb.Ctx(cfg)
    else:
        ctx = default_ctx()

    return tiledb.from_numpy(uri, array, ctx=ctx)


def empty_like(uri, arr, config=None, key=None, tile=None):
    """
    Create and return an empty, writeable DenseArray with schema based on
    a NumPy-array like object.

    :param uri:
    :param arr: NumPy ndarray, or shape tuple
    :param ctx:
    :param kw:
    :return:
    """
    if config:
        cfg = tiledb.Config(config)
        ctx = tiledb.Ctx(cfg)
    else:
        ctx = default_ctx()

    if arr is ArraySchema:
        schema = arr
    else:
        schema = schema_like(arr, tile=tile, ctx=ctx)

    tiledb.DenseArray.create(uri, key=key, schema=schema)
    return tiledb.DenseArray(uri, mode='w', key=key, ctx=ctx)


def from_numpy(uri, array, ctx=default_ctx(), **kw):
    """
    Convenience method, see `tiledb.DenseArray.from_numpy`
    """
    if not isinstance(array, np.ndarray):
        raise Exception("from_numpy is only currently supported for numpy.ndarray")

    return DenseArray.from_numpy(uri, array, ctx=ctx, **kw)

